@@ -2,7 +2,6 @@
 
 from pylab import *
 
-figure(1)
 t = arange(0.0, 1.0+0.01, 0.01)
 s = cos(2*2*pi*t)
 plot(t, s)
@@ -11,7 +10,7 @@ xlabel('time (s)')
 ylabel('voltage (mV)')
 title('About as simple as it gets, folks')
 grid(True)
-#axis([0,1,-1,1])
-savefig('simple_plot')
+savefig('simple_plot.png')
+savefig('simple_plot.ps')
 
 show()
