@@ -28,7 +28,7 @@ ylabel(r'\it{voltage (mV)}',fontsize=16)
 title(r"\TeX\ is Number $\displaystyle\sum_{n=1}^\infty\frac{-e^{i\pi}}{2^n}$!", 
       fontsize=16, color='r')
 grid(True)
-savefig('tex_demo.eps')
+savefig('tex_demo.ps')
 
 
 show()
