@@ -142,9 +142,10 @@ the exception of those in mlab.py provided by matplotlib.
 """
 from __future__ import generators
 
-__version__  = '0.82'
-__revision__ = '$Revision: 1.65 $'
-__date__     = '$Date: 2005/06/15 18:27:37 $'
+
+__version__  = '0.83.2'
+__revision__ = '$Revision: 1.72 $'
+__date__     = '$Date: 2005/07/26 20:20:51 $'
 
 import sys, os, warnings
 import distutils.sysconfig
@@ -159,13 +160,13 @@ The default file location is given in the following order
 
   - environment variable MATPLOTLIBRC
 
-  - HOME/.matplotlibrc if HOME is defined
+  - HOME/.matplotlib/matplotlibrc if HOME is defined
 
-  - PATH/.matplotlibrc where PATH is the return value of
+  - PATH/matplotlibrc where PATH is the return value of
     get_data_path()
 """
 
-import sys, os
+import sys, os, tempfile
 
     
 major, minor1, minor2, s, tmp = sys.version_info
@@ -186,6 +187,21 @@ if not _python23:
              yield i, seq[i]
 
 
+
+def _is_writable_dir(p):
+    """
+    p is a string pointing to a putative writable dir -- return True p
+    is such a string, else False
+    """
+    try: p + ''  # test is string like
+    except TypeError: return False    
+    try:
+        t = tempfile.TemporaryFile(dir=p)
+        t.write('1')
+        t.close()
+    except OSError: return False
+    else: return True
+
 class Verbose:
     """
     A class to handle reporting.  Set the fileo attribute to any file
@@ -259,23 +275,65 @@ class Verbose:
         
 verbose=Verbose('silent')  
 
-def get_home():
-    """
-    return the users HOME dir across platforms or None.
-    
-    On win32, if either HOME is not set or HOME is set but doesn't
-    exist, the value of USERPROFILE will be used instead.
+
+def _get_home():
+    """Return the closest possible equivalent to a 'home' directory.
+
+    We first try $HOME.  Absent that, on NT it's USERPROFILE if
+    defined, else, $HOMEDRIVE\$HOMEPATH, else C:\
+
     """
 
-    if os.environ.has_key('HOME'):
-        path = os.environ['HOME']
-        if os.path.exists(path): return path
+     
+    try: return os.environ['HOME']
+    except KeyError: pass
 
-    if sys.platform=='win32' and os.environ.has_key('USERPROFILE'):
-        path = os.environ['USERPROFILE']
-        if os.path.exists(path): return path
+    if os.name == 'nt':  
+        # For some strange reason, win9x returns 'nt' for os.name.
 
-    return None
+        try: p = os.environ['USERPROFILE']
+        except KeyError: pass
+        else:
+            if os.path.exists(p): return p
+        
+        try: p =  os.path.join(os.environ['HOMEDRIVE'],os.environ['HOMEPATH'])
+        except KeyError: pass
+        else:
+            if os.path.exists(p): return p
+
+        try:
+            import _winreg as wreg
+            key = wreg.OpenKey(wreg.HKEY_CURRENT_USER,
+                               'Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders')
+            p = wreg.QueryValueEx(key,'Personal')[0]
+            key.Close()
+            if os.path.exists(p): return p
+        except: pass
+        return 'C:\\'
+
+    raise RuntimeError('please define environment variable $HOME')
+
+
+
+
+get_home = verbose.wrap('$HOME=%s', _get_home, always=False)
+
+def _get_configdir():
+    """
+    Return the string representing the configuration dir.  If s is the
+    special string _default_, use HOME/.matplotlib.  s must be writable
+    """
+    h = get_home()
+    if not _is_writable_dir(h):
+        raise RuntimeError("'%s' is not a writable dir; you must set environment variable HOME to be a writable dir "%h)
+
+    p = os.path.join(get_home(), '.matplotlib')
+
+    if not _is_writable_dir(p):
+        os.mkdir(p)
+
+    return p
+get_configdir = verbose.wrap('CONFIGDIR=%s', _get_configdir, always=False)
 
 
 def _get_data_path():
@@ -320,6 +378,7 @@ def _get_data_path():
 get_data_path = verbose.wrap('matplotlib data path %s', _get_data_path, always=False)
     
 
+
 def validate_path_exists(s):
     'If s is a path, return s, else False'
     if os.path.exists(s): return s
@@ -594,7 +653,7 @@ defaultParams = {
 
     'grid.color'       :   ['k', validate_color],       # grid color
     'grid.linestyle'   :   [':', str],       # dotted
-    'grid.linewidth'   :   ['0.5', validate_float],     # in points            
+    'grid.linewidth'   :   [0.5, validate_float],     # in points            
 
 
     # figure props
@@ -628,17 +687,9 @@ defaultParams = {
     }
 
 
-def matplotlib_fname():
+def _old_matplotlib_fname():
     """
-    Return the path to the rc file
-
-    Search order:
-
-     * current working dir
-     * environ var MATPLOTLIBRC
-     * HOME/.matplotlibrc
-     * MATPLOTLIBDATA/.matplotlibrc
-     
+    Return the path to the rc file using the old search method
     """
 
     fname = os.path.join( os.getcwd(), '.matplotlibrc')
@@ -663,6 +714,60 @@ def matplotlib_fname():
         warnings.warn('Could not find .matplotlibrc; using defaults')
     return fname
 
+def matplotlib_fname():
+    """
+    Return the path to the rc file
+
+    Search order:
+
+     * current working dir
+     * environ var MATPLOTLIBRC
+     * HOME/.matplotlib/matplotlibrc
+     * MATPLOTLIBDATA/matplotlibrc
+     
+
+    """
+
+    oldname = os.path.join( os.getcwd(), '.matplotlibrc')
+    if os.path.exists(oldname):
+        print >> sys.stderr, """\
+WARNING: Old rc filename ".matplotlibrc" found in working dir
+  and and renamed to new default rc file name "matplotlibrc"
+  (no leading"dot"). """
+        os.rename('.matplotlibrc', 'matplotlibrc')
+
+    home = get_home()
+    oldname = os.path.join( home, '.matplotlibrc')
+    if os.path.exists(oldname):
+        configdir = get_configdir()
+        newname = os.path.join(configdir, 'matplotlibrc')
+        print >> sys.stderr, """\
+WARNING: Old rc filename "%s" found and renamed to
+  new default rc file name "%s"."""%(oldname, newname)
+
+        os.rename(oldname, newname)
+            
+        
+    fname = os.path.join( os.getcwd(), 'matplotlibrc')
+    if os.path.exists(fname): return fname
+
+    if os.environ.has_key('MATPLOTLIBRC'):
+        path =  os.environ['MATPLOTLIBRC']
+        if os.path.exists(path):
+            fname = os.path.join(path, 'matplotlibrc')
+            if os.path.exists(fname):
+                return fname
+
+    fname = os.path.join(get_configdir(), 'matplotlibrc')
+    if os.path.exists(fname): return fname
+    
+    
+    path =  get_data_path() # guaranteed to exist or raise
+    fname = os.path.join(path, 'matplotlibrc')
+    if not os.path.exists(fname):
+        warnings.warn('Could not find matplotlibrc; using defaults')
+    return fname
+
 
 
 def rc_params():
@@ -699,7 +804,7 @@ def rc_params():
         key = key.strip()
         if key in deprecated_map.keys():
             alt = deprecated_map[key]
-            warnings.warn('%s is deprecated in .matplotlibrc - use %s instead.' % (key, alt))
+            warnings.warn('%s is deprecated in matplotlibrc - use %s instead.' % (key, alt))
             key = alt
             
         if not defaultParams.has_key(key):
@@ -809,6 +914,12 @@ def rcdefaults():
     rcParams.update(rcParamsDefault)
 
 
+    
+    
+        
+    
+
+
 
 # Now allow command line to override
 
@@ -816,9 +927,9 @@ def rcdefaults():
 # flag)
 
 _knownBackends = {
-    'Agg2':1, 'Agg':1, 'Cairo':1, 'FltkAgg':1, 'GD':1, 'GDK':1, 'GTK':1, 'GTKAgg':1,
-    'GTKCairo':1, 'Paint':1, 'PS':1, 'LaTeX':1, 'QtAgg':1, 'SVG':1, 'Template':1,
-    'TkAgg':1, 'WX':1, 'WXAgg':1, }
+    'Agg2':1, 'Agg':1, 'Cairo':1, 'CocoaAgg':1, 'FltkAgg':1, 'GD':1, 'GDK':1,
+    'GTK':1, 'GTKAgg':1, 'GTKCairo':1, 'Paint':1, 'PS':1, 'LaTeX':1, 'QtAgg':1,
+    'SVG':1, 'Template':1, 'TkAgg':1, 'WX':1, 'WXAgg':1, }
 
 
 known = _knownBackends.keys()
@@ -875,18 +986,3 @@ verbose.report('verbose.level %s'%verbose.level)
 verbose.report('interactive is %s'%rcParams['interactive'])
 verbose.report('platform is %s'%sys.platform)
 verbose.report('loaded modules: %s'%sys.modules.keys(), 'debug')
-
-# True if we are running under Py2EXE (or similar)
-FROZEN = hasattr(sys, 'frozen')
-
-
-class MPLError (Exception):
-    """Exception for Matplotlib errors
-    Attributes:
-        message -- explanation of the error
-    """
-    def __init__ (self, message):
-        self.message = message
-
-    def __str__ (self):
-        return self.message
