@@ -71,6 +71,9 @@ BUILT_GDK       = False
 
 AGG_VERSION = 'agg23'
 
+# for nonstandard installation/build with --prefix variable
+numarray_inc_dirs = ['']
+
 class CleanUpFile:
     """CleanUpFile deletes the specified filename when self is destroyed."""
     def __init__(self, name):
@@ -115,7 +118,9 @@ def add_agg_flags(module):
     module.libraries.append('png')
     module.libraries.append('z')
     add_base_flags(module)
-    module.include_dirs.extend(['src','%s/include'%AGG_VERSION, '.'])
+    module.include_dirs.extend(['src','swig', '%s/include'%AGG_VERSION, '.'])
+    
+
 
     # put these later for correct link order
     module.libraries.extend(['stdc++', 'm'])
@@ -351,7 +356,7 @@ def build_ft2font(ext_modules, packages):
 def build_gtkagg(ext_modules, packages, numerix):
     global BUILT_GTKAGG
     if BUILT_GTKAGG: return # only build it if you you haven't already
-    deps = ['src/_gtkagg.cpp', 'src/mplutils.cpp']
+    deps = ['src/_gtkagg.cpp', 'src/mplutils.cpp', 'src/_transforms.cpp']
     deps.extend(glob.glob('CXX/*.cxx'))
     deps.extend(glob.glob('CXX/*.c'))
 
@@ -412,8 +417,8 @@ def build_agg(ext_modules, packages, numerix):
         deps.append('src/_na_backend_agg.cpp')
         module = Extension(
             'matplotlib.backends._na_backend_agg',
-            deps
-            ,
+            deps,
+            include_dirs=numarray_inc_dirs,
             )    
         module.extra_compile_args.append('-DNUMARRAY=1')
         add_agg_flags(module)
@@ -459,8 +464,8 @@ def build_image(ext_modules, packages, numerix):
         deps.extend(glob.glob('CXX/*.c'))
         module = Extension(
             'matplotlib._na_image',
-            deps
-            ,
+            deps,
+            include_dirs=numarray_inc_dirs,
             )    
         module.extra_compile_args.append('-DNUMARRAY=1')
         add_agg_flags(module)
@@ -517,7 +522,7 @@ def build_transforms(ext_modules, packages, numerix):
                              ['src/_na_transforms.cpp',
                               'src/mplutils.cpp'] + cxx,
                              libraries = ['stdc++', 'm'],
-                             include_dirs = ['src', '.'],
+                             include_dirs = ['src', '.']+numarray_inc_dirs,
                              )
         
         module.extra_compile_args.append("-DNUMARRAY=1")
@@ -564,6 +569,7 @@ def build_contour(ext_modules, packages, numerix):
             'matplotlib._na_cntr',
             [  'src/_na_cntr.c',],
             #libraries = ['stdc++'],
+            include_dirs=numarray_inc_dirs,
             )
         module.extra_compile_args.append('-DNUMARRAY=1')
         add_base_flags(module)
@@ -590,10 +596,10 @@ def build_gdk(ext_modules, packages, numerix):
     if numerix in ["numarray","both"]: # Build for numarray
         temp_copy('src/_backend_gdk.c', 'src/_na_backend_gdk.c')
         module = Extension(
-            'matplotlib._na_backend_gdk',
-		[ 'src/_na_backend_gdk.c',
-		],
-		libraries = [],
+            'matplotlib.backends._na_backend_gdk',
+            ['src/_na_backend_gdk.c', ],
+            libraries = [],
+            include_dirs=numarray_inc_dirs,
             )
         module.extra_compile_args.append('-DNUMARRAY=1')
         add_base_flags(module)
@@ -603,10 +609,9 @@ def build_gdk(ext_modules, packages, numerix):
     if numerix in ["Numeric","both"]: # Build for Numeric
         temp_copy('src/_backend_gdk.c', 'src/_nc_backend_gdk.c')
         module = Extension(
-            'matplotlib._nc_backend_gdk',
-		[ 'src/_nc_backend_gdk.c',
-		],
-		libraries = [],
+            'matplotlib.backends._nc_backend_gdk',
+            ['src/_nc_backend_gdk.c', ],
+            libraries = [],
             )
         module.extra_compile_args.append('-DNUMERIC=1')
         add_base_flags(module)
