@@ -8,6 +8,7 @@
 #include <stdexcept>
 #include <png.h>
 
+
 #include "agg_conv_transform.h"
 #include "agg_conv_curve.h"
 #include "agg_scanline_storage_aa.h"
@@ -30,16 +31,18 @@
 #endif   
 
 /* ------------ RendererAgg methods ------------- */
-
+ 
 
 GCAgg::GCAgg(const Py::Object &gc, double dpi) : 
   dpi(dpi), isaa(true), linewidth(1.0), alpha(1.0), cliprect(NULL), 
   Ndash(0), dashOffset(0.0), dasha(NULL)
 {
   
+  _VERBOSE("GCAgg::GCAgg");
   linewidth = points_to_pixels ( gc.getAttr("_linewidth") ) ;
   alpha = Py::Float( gc.getAttr("_alpha") );
   color = get_color(gc);
+  _set_antialiased(gc);
   _set_linecap(gc);
   _set_joinstyle(gc);
   _set_dashes(gc);
@@ -81,11 +84,11 @@ GCAgg::_set_linecap(const Py::Object& gc) {
   std::string capstyle = Py::String( gc.getAttr( "_capstyle" ) );
   
   if (capstyle=="butt") 
-    cap = agg::vcgen_stroke::butt_cap;
+    cap = agg::butt_cap;
   else if (capstyle=="round") 
-    cap = agg::vcgen_stroke::round_cap;
+    cap = agg::round_cap;
   else if(capstyle=="projecting") 
-    cap = agg::vcgen_stroke::square_cap;
+    cap = agg::square_cap;
   else 
     throw Py::ValueError(Printf("GC _capstyle attribute must be one of butt, round, projecting; found %s", capstyle.c_str()).str());
   
@@ -98,11 +101,11 @@ GCAgg::_set_joinstyle(const Py::Object& gc) {
   std::string joinstyle = Py::String( gc.getAttr("_joinstyle") );
   
   if (joinstyle=="miter") 
-    join =  agg::vcgen_stroke::miter_join;
+    join =  agg::miter_join;
   else if (joinstyle=="round") 
-    join = agg::vcgen_stroke::round_join;
+    join = agg::round_join;
   else if(joinstyle=="bevel") 
-    join = agg::vcgen_stroke::bevel_join;
+    join = agg::bevel_join;
   else 
     throw Py::ValueError(Printf("GC _joinstyle attribute must be one of butt, round, projecting; found %s", joinstyle.c_str()).str());
   
@@ -146,16 +149,17 @@ void
 GCAgg::_set_clip_rectangle( const Py::Object& gc) {
   //set the clip rectangle from the gc
   
-  _VERBOSE("GCAgg::set_clip_rectangle");
+  _VERBOSE("GCAgg::_set_clip_rectangle");
 
   delete [] cliprect;
   cliprect = NULL;
+
   
   Py::Object o ( gc.getAttr( "_cliprect" ) );
-  if (o.ptr()==Py_None) 
+  if (o.ptr()==Py_None) {
     return;
-  
-  
+  } 
+
   Py::SeqBase<Py::Object> rect( o );
   
   double l = Py::Float(rect[0]) ; 
@@ -164,15 +168,15 @@ GCAgg::_set_clip_rectangle( const Py::Object& gc) {
   double h = Py::Float(rect[3]) ; 
   
   cliprect = new double[4];
+  //todo check for memory alloc failure
   cliprect[0] = l;
   cliprect[1] = b;
   cliprect[2] = w;
   cliprect[3] = h;
-  
 }
 
 
-
+ 
 
 
 const size_t 
@@ -191,6 +195,8 @@ RendererAgg::RendererAgg(unsigned int width, unsigned int height, double dpi,
   
   
   pixBuffer = new agg::int8u[NUMBYTES];  
+  cacheBuffer = NULL;
+
   
   renderingBuffer = new agg::rendering_buffer;
   renderingBuffer->attach(pixBuffer, width, height, stride);
@@ -208,6 +214,8 @@ RendererAgg::RendererAgg(unsigned int width, unsigned int height, double dpi,
   
 };
 
+
+
 void
 RendererAgg::set_clipbox_rasterizer( double *cliprect) {
   //set the clip rectangle from the gc
@@ -563,6 +571,86 @@ RendererAgg::draw_line_collection(const Py::Tuple& args) {
 }
 
 
+
+Py::Object
+RendererAgg::copy_from_bbox(const Py::Tuple& args) {
+  //copy region in bbox to buffer and return swig/agg buffer object
+  args.verify_length(1);
+   
+
+  agg::rect r = bbox_to_rect(args[0]);
+  /*
+    r.x1 -=5;
+    r.y1 -=5;
+    r.x2 +=5;
+    r.y2 +=5;
+  */
+  int boxwidth = r.x2-r.x1;
+  int boxheight = r.y2-r.y1;
+  int boxstride = boxwidth*4;
+  agg::buffer buf(boxwidth, boxheight, boxstride, false);
+  if (buf.data ==NULL) {
+    throw Py::MemoryError("RendererAgg::copy_from_bbox could not allocate memory for buffer");
+  }
+    
+  agg::rendering_buffer rbuf;
+  rbuf.attach(buf.data, boxwidth, boxheight, boxstride);
+
+  pixfmt pf(rbuf);
+  renderer_base rb(pf);
+  //rb.clear(agg::rgba(1, 0, 0)); //todo remove me
+  rb.copy_from(*renderingBuffer, &r, -r.x1, -r.y1);
+  BufferRegion* reg = new BufferRegion(buf, r);
+
+  return Py::asObject(reg);
+  
+  
+  
+}
+
+Py::Object
+RendererAgg::restore_region(const Py::Tuple& args) {
+  //copy BufferRegion to buffer
+  args.verify_length(1);
+  BufferRegion* region  = static_cast<BufferRegion*>(args[0].ptr());
+  
+  if (region->aggbuf.data==NULL) 
+    return Py::Object();
+  //throw Py::ValueError("Cannot restore_region from NULL data");
+
+    
+  agg::rendering_buffer rbuf;
+  rbuf.attach(region->aggbuf.data, 
+	      region->aggbuf.width, 
+	      region->aggbuf.height, 
+	      region->aggbuf.stride);
+
+  rendererBase->copy_from(rbuf, 0, region->rect.x1, region->rect.y1);
+
+  return Py::Object();
+  
+  
+  
+}
+
+
+agg::rect_base<int>
+RendererAgg::bbox_to_rect(const Py::Object& o) {
+  //return the agg::rect for bbox, flipping y
+
+  Bbox* clipbox = static_cast<Bbox*>(o.ptr());
+  double l = clipbox->ll_api()->x_api()->val() ; 
+  double b = clipbox->ll_api()->y_api()->val();
+  double r = clipbox->ur_api()->x_api()->val() ; 
+  double t = clipbox->ur_api()->y_api()->val() ; ;       
+  
+  agg::rect rect( (int)l, height-(int)t, (int)r, height-(int)b ) ;
+  if (!rect.is_valid())
+    throw Py::ValueError("Invalid rectangle in bbox_to_rect");
+  return rect;
+  
+}
+
 void
 RendererAgg::set_clip_from_bbox(const Py::Object& o) {
   if (o.ptr() != Py_None) {  //using clip
@@ -913,6 +1001,7 @@ RendererAgg::draw_lines(const Py::Tuple& args) {
   _VERBOSE("RendererAgg::draw_lines");
   args.verify_length(4);  
   GCAgg gc = GCAgg(args[0], dpi);
+  
   set_clipbox_rasterizer(gc.cliprect);
   
   Py::Object xo = args[1];
@@ -961,6 +1050,8 @@ RendererAgg::draw_lines(const Py::Tuple& args) {
   
   double lastx(-2.0), lasty(-2.0);
   
+  bool snapto = Nx==2;
+  
   for (size_t i=0; i<Nx; i++) {
     thisx = *(double *)(xa->data + i*xa->strides[0]);
     thisy = *(double *)(ya->data + i*ya->strides[0]);
@@ -983,9 +1074,14 @@ RendererAgg::draw_lines(const Py::Tuple& args) {
     if (!moveto && (i>0) && fabs(thisx-lastx)<1.0 && fabs(thisy-lasty)<1.0) {
       continue;
     }
-    
+
     lastx = thisx;
     lasty = thisy;
+    if (snapto) {
+      thisx = (int)thisx + 0.5;
+      thisy = (int)thisy + 0.5;
+    }
+
     if (moveto)
       path.move_to(thisx, thisy);
     else
@@ -998,6 +1094,8 @@ RendererAgg::draw_lines(const Py::Tuple& args) {
   Py_XDECREF(xa);
   Py_XDECREF(ya);
   
+  //typedef agg::conv_transform<agg::path_storage, agg::trans_affine> path_t;
+  //path_t transpath(path, xytrans);
   _render_lines_path(path, gc);
   
   _VERBOSE("RendererAgg::draw_lines DONE");
@@ -1005,10 +1103,11 @@ RendererAgg::draw_lines(const Py::Tuple& args) {
   
 }
 
+template<class PathSource>
 void 
-RendererAgg::_render_lines_path(agg::path_storage &path, const GCAgg& gc) {
+RendererAgg::_render_lines_path(PathSource &path, const GCAgg& gc) {
   
-  typedef agg::path_storage path_t;
+  typedef PathSource path_t;
   //typedef agg::conv_transform<agg::path_storage, agg::trans_affine> path_t;
   typedef agg::conv_stroke<path_t> stroke_t;
   typedef agg::conv_dash<path_t> dash_t;
@@ -1050,6 +1149,7 @@ RendererAgg::_render_lines_path(agg::path_storage &path, const GCAgg& gc) {
   
 }
 
+/*
 Py::Object
 RendererAgg::draw_markers(const Py::Tuple& args) {
   // there is a win32 specific segfault that happens when using the
@@ -1065,6 +1165,7 @@ RendererAgg::draw_markers(const Py::Tuple& args) {
 #endif
 }
 
+
 Py::Object
 RendererAgg::_draw_markers_nocache(const Py::Tuple& args) {
   //draw_markers(gc, path, rgbFace, xo, yo, transform)
@@ -1167,6 +1268,9 @@ RendererAgg::_draw_markers_nocache(const Py::Tuple& args) {
     
     
     thisy = heightd - thisy;  //flipy
+
+    thisx = (int)thisx + 0.5;
+    thisy = (int)thisy + 0.5;
     markers.move_to(thisx, thisy);
     //agg::path_storage marker;
     double x, y;
@@ -1186,9 +1290,9 @@ RendererAgg::_draw_markers_nocache(const Py::Tuple& args) {
   
 }
 
-
+*/
 Py::Object
-RendererAgg::_draw_markers_cache(const Py::Tuple& args) {
+RendererAgg::draw_markers(const Py::Tuple& args) {
   //_draw_markers_cache(gc, path, rgbFace, xo, yo, transform)
   theRasterizer->reset_clipping();
   
@@ -1203,7 +1307,7 @@ RendererAgg::_draw_markers_cache(const Py::Tuple& args) {
   if (SWIG_ConvertPtr(args[1].ptr(),(void **)(&ppath), descr, 0) == -1) 
     throw Py::TypeError("Could not convert path_storage");
   facepair_t face = _get_rgba_face(args[2], gc.alpha);
-  
+
   Py::Object xo = args[3];
   Py::Object yo = args[4];
   
@@ -1245,7 +1349,6 @@ RendererAgg::_draw_markers_cache(const Py::Tuple& args) {
   typedef agg::conv_curve<agg::path_storage> curve_t;
   curve_t curve(*ppath);
 
-
   //maxim's suggestions for cached scanlines
   agg::scanline_storage_aa8 scanlines;  
   theRasterizer->reset();
@@ -1260,7 +1363,7 @@ RendererAgg::_draw_markers_cache(const Py::Tuple& args) {
     scanlines.serialize(fillCache);
   }
   
-  
+
   agg::conv_stroke<curve_t> stroke(curve);
   stroke.width(gc.linewidth);
   stroke.line_cap(gc.cap);
@@ -1272,8 +1375,6 @@ RendererAgg::_draw_markers_cache(const Py::Tuple& args) {
   agg::int8u* strokeCache = new agg::int8u[strokeSize]; // or any container
   scanlines.serialize(strokeCache);
   
-  
-  
   theRasterizer->reset_clipping();
   
   
@@ -1287,7 +1388,7 @@ RendererAgg::_draw_markers_cache(const Py::Tuple& args) {
     int h = (int)(gc.cliprect[3]) ;     
     rendererBase->clip_box(l, height-(b+h),l+w, height-b);
   }
-  
+
   double thisx, thisy;
   for (size_t i=0; i<Nx; i++) {
     thisx = *(double *)(xa->data + i*xa->strides[0]);
@@ -1302,9 +1403,11 @@ RendererAgg::_draw_markers_cache(const Py::Tuple& args) {
       }
     
     xytrans.transform(&thisx, &thisy);
-    
-    
+
     thisy = heightd - thisy;  //flipy
+
+    thisx = (int)thisx + 0.5;
+    thisy = (int)thisy + 0.5;
     
     agg::serialized_scanlines_adaptor_aa8 sa;
     agg::serialized_scanlines_adaptor_aa8::embedded_scanline sl;
@@ -1321,20 +1424,22 @@ RendererAgg::_draw_markers_cache(const Py::Tuple& args) {
     rendererAA->color(gc.color);          
     agg::render_scanlines(sa, sl, *rendererAA);
     
-  } //for each marker
-  
+  } //for each marker 
+
   Py_XDECREF(xa);
   Py_XDECREF(ya);
   
   if (face.first) 
     delete [] fillCache;
   delete [] strokeCache;
-  
   return Py::Object();
-  
+   
 }
 
-Py::Object
+
+
+ 
+Py::Object 
 RendererAgg::draw_path(const Py::Tuple& args) {
   //draw_path(gc, rgbFace, path, transform)
   theRasterizer->reset_clipping();
@@ -1360,8 +1465,8 @@ RendererAgg::draw_path(const Py::Tuple& args) {
   }
   catch(...) {
     throw Py::ValueError("Domain error on affine_params_api in RendererAgg::draw_path");  
-  }
-  
+  } 
+   
   agg::trans_affine xytrans = agg::trans_affine(a,b,c,d,tx,ty);  
   
   double heightd = double(height);  
@@ -1404,10 +1509,17 @@ RendererAgg::draw_text(const Py::Tuple& args) {
   
   
   FT2Font *font = static_cast<FT2Font*>(args[0].ptr());
-  
-  int x = Py::Int( args[1] );
-  int y = Py::Int( args[2] );
-  
+
+  int x(0),y(0);
+  try {
+    x = Py::Int( args[1] );
+    y = Py::Int( args[2] );
+  }
+  catch (Py::TypeError) {
+    //x,y out of range; todo issue warning?
+    return Py::Object();
+  }
+   
   GCAgg gc = GCAgg(args[3], dpi);
   
   set_clipbox_rasterizer( gc.cliprect);
@@ -1806,6 +1918,7 @@ RendererAgg::~RendererAgg() {
   delete pixFmt;
   delete renderingBuffer;
   delete [] pixBuffer;
+  delete [] cacheBuffer;
     
 }
 
@@ -1873,6 +1986,12 @@ void RendererAgg::init_type()
 		     "buffer = buffer_rgba()");
   add_varargs_method("clear", &RendererAgg::clear, 
 		     "clear()"); 
+  add_varargs_method("copy_from_bbox", &RendererAgg::copy_from_bbox, 
+		     "copy_from_bbox(bbox)"); 
+
+  add_varargs_method("restore_region", &RendererAgg::restore_region, 
+		     "restore_region(region)"); 
+
   
 }
 
